@@ -20,6 +20,16 @@ from zope.globalrequest import getRequest
 from zope.lifecycleevent import ObjectCreatedEvent
 from zExceptions import NotFound
 from kitconcept.contentcreator.scales import plone_scale_generate_on_save
+from plone.restapi.behaviors import IBlocks
+from plone.app.content.interfaces import INameFromTitle
+from zope.container.interfaces import INameChooser
+
+from plone.restapi.interfaces import IDeserializeFromJson
+from zope.component import getMultiAdapter
+from plone.dexterity.utils import iterSchemata
+from plone.restapi.interfaces import IFieldSerializer
+from plone.restapi.services.content.utils import add
+from plone.restapi.services.content.utils import create
 
 import json
 import logging
@@ -37,16 +47,6 @@ except pkg_resources.DistributionNotFound:  # pragma: no restapi
     ARCHETYPES_PRESENT = False
 
 
-try:
-    pkg_resources.get_distribution("plone.restapi")
-    from plone.restapi.interfaces import IDeserializeFromJson
-    from plone.restapi.services.content.utils import add
-    from plone.restapi.services.content.utils import create
-
-    PRESTAPI_PRESENT = True
-except pkg_resources.DistributionNotFound:  # pragma: no restapi
-    PRESTAPI_PRESENT = False
-
 try:
     from Products.CMFPlone.utils import get_installer
 except ImportError:
@@ -499,6 +499,8 @@ def create_item_runner(  # noqa
             if temporarily_wrapped:
                 obj = aq_base(obj)
 
+            path = "/".join(obj.getPhysicalPath())
+
             if create_object:
                 if not getattr(deserializer, "notifies_create", False):
                     notify(ObjectCreatedEvent(obj))
@@ -545,7 +547,11 @@ def create_item_runner(  # noqa
 
             id_ = obj.id  # get the real id
             path = "/".join(obj.getPhysicalPath())
-            logger.info("{0}: created".format(path))
+
+            if create_object:
+                logger.info("{0}: created".format(path))
+            else:
+                logger.info("{0}: edited".format(path))
 
             # CONSTRAIN TYPES
             locally_allowed_types = opts.get("locally_allowed_types", False)
@@ -600,6 +606,116 @@ def create_item_runner(  # noqa
         )
 
 
+def get_objects_created(container, content_structure):
+    paths = []
+
+    for data in content_structure:
+        id_ = data.get("id", None)
+        obj = container[id_]
+
+        paths.append("/".join(obj.getPhysicalPath()))
+        result = get_objects_created(obj, content_structure=data.get("items", []))
+
+        if result:
+            paths = paths + result
+        return paths
+
+
+def refresh_objects_created_by_structure(container, content_structure):
+    def deserialize(obj, blocks=None, validate_all=False):
+        request = getRequest()
+        request["BODY"] = json.dumps({"blocks": blocks})
+        deserializer = getMultiAdapter((obj, request), IDeserializeFromJson)
+
+        return deserializer(validate_all=validate_all)
+
+    def serialize(context):
+        request = getRequest()
+        fieldname = "blocks"
+        for schema in iterSchemata(context):
+            if fieldname in schema:
+                field = schema.get(fieldname)
+                break
+
+        serializer = getMultiAdapter((field, context, request), IFieldSerializer)
+        return serializer()
+
+    def get_content_by_data(data, container):
+        type_ = data.get("@type", None)
+        id_ = data.get("id", None)
+        title = data.get("title", None)
+
+        obj = create(container, type_, id_=id_, title=title)
+        chooser = INameChooser(container)
+        # INameFromTitle adaptable objects should not get a name
+        # suggestion. NameChooser would prefer the given name instead of
+        # the one provided by the INameFromTitle adapter.
+        suggestion = None
+        name_from_title = INameFromTitle(obj, None)
+        if name_from_title is None:
+            suggestion = obj.Title()
+        id_ = chooser.chooseName(suggestion, obj)
+        original = id_[:-2]
+        return container.get(original, None)
+
+    for data in content_structure:
+        id_ = data.get("id", None)
+        if not id_:
+            obj = get_content_by_data(data, container)
+            if not obj:
+                print_error(
+                    "id can't be guessed for {0} in container {1}".format(
+                        "/".join(container.getPhysicalPath()), data["title"]
+                    )
+                )
+                return
+        else:
+            obj = container.get(id_, None)
+
+        if obj and IBlocks.providedBy(obj):
+            blocks_serialized = serialize(obj)
+            deserialize(obj, blocks_serialized)
+
+        if obj:
+            refresh_objects_created_by_structure(
+                obj, content_structure=data.get("items", [])
+            )
+
+
+def refresh_objects_created_by_file(filepath, file_):
+    def deserialize(obj, blocks=None, validate_all=False):
+        request = getRequest()
+        request["BODY"] = json.dumps({"blocks": blocks})
+        deserializer = getMultiAdapter((obj, request), IDeserializeFromJson)
+
+        return deserializer(validate_all=validate_all)
+
+    def serialize(context):
+        request = getRequest()
+        fieldname = "blocks"
+        for schema in iterSchemata(context):
+            if fieldname in schema:
+                field = schema.get(fieldname)
+                break
+
+        serializer = getMultiAdapter((field, context, request), IFieldSerializer)
+        return serializer()
+
+    splitted_path = os.path.splitext(file_)[0].split(".")
+    path = "/" + "/".join(splitted_path[:-1])
+    id_ = splitted_path[-1]
+    try:
+        container = api.content.get(path=path)
+    except NotFound:
+        print_error('Could not look up container under "{}"'.format(path))
+        return
+
+    obj = container.get(id_, None)
+    if obj and IBlocks.providedBy(obj):
+        blocks_serialized = serialize(obj)
+        deserialize(obj, blocks_serialized)
+
+
 def content_creator_from_folder(
     folder_name=os.path.join(os.path.dirname(__file__), "content_creator"),
     base_image_path=os.path.join(os.path.dirname(__file__), "content_images"),
@@ -661,10 +777,14 @@ def content_creator_from_folder(
         )
 
     files = sorted(os.listdir(folder), key=sort_key)
+    has_content_json = False
+    # has_siteroot_json = False
+
     for file_ in files:
         # If a content.json is found, proceed as if it contains a normal json arrayed
         # structure
         if file_ == "content.json":
+            has_content_json = True
             print_info("content.json file found, creating content")
             content_structure = load_json(os.path.join(folder, "content.json"))
             create_item_runner(
@@ -717,6 +837,18 @@ def content_creator_from_folder(
         except Exception as e:  # noqa
             print_error('Error in file structure: "{0}": {1}'.format(filepath, e))
 
+    # After creation, we refresh all the content created to update resolveuids
+    if len(files) > 0:
+        print_info("Refreshing content serialization after creation...")
+    for file_ in files:
+        filepath = os.path.join(folder, file_)
+        refresh_objects_created_by_file(filepath, file_)
+    if has_content_json:
+        print_info(
+            "Refreshing structured (content.json) content serialization after creation..."
+        )
+        refresh_objects_created_by_structure(api.portal.get(), content_structure)
+
     for content_type in temp_enable_content_types:
         disable_content_type(portal, content_type)
 
@@ -729,11 +861,11 @@ def modify_siteroot(root_info):
     if not getattr(portal, "blocks", False):
         portal.manage_addProperty("blocks", json.dumps(blocks), "string")
     else:
-        portal.blocks = blocks
+        portal.blocks = json.dumps(blocks)
 
     if not getattr(portal, "blocks_layout", False):
         portal.manage_addProperty(
             "blocks_layout", json.dumps(blocks_layout), "string"
         )  # noqa
     else:
-        portal.blocks_layout = blocks_layout
+        portal.blocks_layout = json.dumps(blocks_layout)
